/* -*-c-*-
 *
 * Identifies and logs the client of a connection
 *
 * (c) 1999 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the `fw' port forwarder.
 *
 * `fw' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * `fw' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with `fw'; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Header files ------------------------------------------------------*/

#include "config.h"

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <syslog.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <mLib/alloc.h>
#include <mLib/bres.h>
#include <mLib/conn.h>
#include <mLib/dstr.h>
#include <mLib/ident.h>
#include <mLib/report.h>
#include <mLib/sel.h>
#include <mLib/selbuf.h>
#include <mLib/str.h>

#include "fw.h"
#include "identify.h"

/*----- Magic numbers -----------------------------------------------------*/

#define TIMEOUT 15			/* Seconds to wait for answers */

/*----- Data structures ---------------------------------------------------*/

/* --- Structure to track the progress of an identification --- */

typedef struct id {
  id_req q;				/* Copy of client's request block */
  time_t when;				/* When the connection occurred */
  conn c;				/* Connection selector */
  unsigned state;			/* Current state of the world */
  sel_timer t;				/* Timeout selector */
  bres_client r;			/* Backgd resolver client block */
  ident_request i;			/* Ident client block */
  char host[128];			/* Resolved hostname */
  char user[64];			/* Authenticated client user */
} id;

#define S_HOST 1u			/* Read the hostname from resolver */
#define S_USER 2u			/* Read the username from RFC931 */
#define S_TIMER 4u			/* Timeout has completed */

/*----- Main code ---------------------------------------------------------*/

/* --- @id_done@ --- *
 *
 * Arguments:	@id *i@ = pointer to identification block
 *
 * Returns:	---
 *
 * Use:		Finishes with an identification block.
 */

static void id_done(id *i)
{
  /* --- Close down the various dependent bits --- */

  if (!(i->state & S_HOST))
    bres_abort(&i->r);
  if (!(i->state & S_USER))
    ident_abort(&i->i);
  if (!(i->state & S_TIMER))
    sel_rmtimer(&i->t);

  /* --- Report the final result --- */

  fw_log(i->when, "[%s] %s from %s@%s [%s:%u]",
	 i->q.desc, i->q.act,
	 i->user, i->host,
	 inet_ntoa(i->q.rsin.sin_addr), (unsigned)ntohs(i->q.rsin.sin_port));

  /* --- Dispose of the block --- */

  REFFD_DEC(i->q.r);
  xfree(i);
}

/* --- @id_res@ --- *
 *
 * Arguments:	@struct hostent *h@ = name of the resolved host
 *		@void *vp@ = pointer to identification block
 *
 * Returns:	---
 *
 * Use:		Responds to a completed reverse name resolution.
 */

static void id_res(struct hostent *h, void *vp)
{
  id *i = vp;
  if (h)
    str_sanitize(i->host, h->h_name, sizeof(i->host));
  i->state |= S_HOST;
  if (i->state & S_USER)
    id_done(i);
}

/* --- @id_ident@ --- *
 *
 * Arguments:	@ident_reply *i@ = pointer to string read from server
 *		@void *vp@ = pointer to identification block
 *
 * Returns:	---
 *
 * Use:		Responds to a line read from the remote RFC931 server.
 */

static void id_ident(ident_reply *ir, void *vp)
{
  id *i = vp;

  /* --- Read the information from the client --- */

  if (ir && ir->type == IDENT_USERID)
    str_sanitize(i->user, ir->u.userid.user, sizeof(i->user));

  /* --- Maybe finish off this identification --- */

  i->state |= S_USER;
  if (i->state & S_HOST)
    id_done(i);
}

/* --- @id_timer@ --- *
 *
 * Arguments:	@struct timeval *tv@ = pointer to the current time
 *		@void *vp@ = pointer to identification block
 *
 * Returns:	---
 *
 * Use:		Times an identification job out.
 */

static void id_timer(struct timeval *tv, void *vp)
{
  id *i = vp;
  i->state |= S_TIMER;
  id_done(i);
}

/* --- @identify@ --- *
 *
 * Arguments:	@const id_req *q@ = pointer to request block
 *
 * Returns:	---
 *
 * Use:		Starts a background ident lookup and reverse-resolve job
 *		which will, eventually, report a message to the system log.
 */

void identify(const id_req *q)
{
  id *i;

  /* --- Initialize the block with stuff --- */

  i = xmalloc(sizeof(*i));
  i->q = *q;
  REFFD_INC(i->q.r);

  str_sanitize(i->host, inet_ntoa(q->rsin.sin_addr), sizeof(i->host));
  strcpy(i->user, "<ANONYMOUS>");
  i->state = 0;
  i->when = time(0);

  /* --- Set up the connection to the identity server --- */

  ident(&i->i, sel, &q->lsin, &q->rsin, id_ident, i);

  /* --- Set up the name resolver --- */

  bres_byaddr(&i->r, q->rsin.sin_addr, id_res, i);

  /* --- Set up the time limiter --- */

  {
    struct timeval tv;
    gettimeofday(&tv, 0);
    tv.tv_sec += TIMEOUT;
    sel_addtimer(sel, &i->t, &tv, id_timer, i);
  }
}

/*----- That's all, folks -------------------------------------------------*/
